import unittest
from array import array,zeros
import itertools 
from test import test_support
import time
import os
import os.path
from weka.core import Debug
from ij import IJ
from classify import classify
import segmentation.WekaClassifier as WekaClassifier
import segmentation.GCSegmenter as GCSegmenter
import ij.process.FloatProcessor as FloatProcessor
import ij.ImageStack as ImageStack
import ij.ImagePlus as ImagePlus
from funsSegm import PixProbFun

import graphcut.Graph_Cut as Graph_Cut
from ParticleAnal import ParticleAnal
import IsletJ.ParticleFilter as ParticleFilter

class SegmentationResults(unittest.TestCase):
    
    classifierFolder = "data"  
    classifierName = "classifier-HSV-RF"
    dataFolder = "data/img"
    testImg = "test_image.jpg"

    def setImgFromVector(self,v, w, h):
        ip = FloatProcessor(w, h, v, None)
        stack = ImageStack(w, h)
        stack.addSlice(ip)
        return ImagePlus('', stack) 

    
    def setUp(self):
        
        classifierFile = os.path.join(os.getcwd(), self.classifierFolder, self.classifierName)
        self.classifier = Debug.loadFromFile(classifierFile)
        self.image = IJ.openImage(os.path.join(os.getcwd(), self.dataFolder, self.testImg))
        t1 = time.time()
        [self.segmentation, self.distribs] = classify(self.image, self.classifier)
        self.pixIslProb, pixBkgProb, pixExoProb = PixProbFun(self.segmentation,self.distribs)
        t2 = time.time()
        self.pyTime = t2-t1

        ip = self.image.getProcessor()
        self.imgW = ip.getWidth()
        self.imgH = ip.getHeight()
        
    def test_WekaClassifier(self):
        classifier = WekaClassifier(self.classifier, self.imgW, self.imgH)
        t1 = time.time()
        segment = classifier.getClassificationVec(self.image)        
        probsImg = classifier.getProbabilities(self.image)
        t2 = time.time()
        print 'Java speedup:', (self.pyTime/(t2-t1)-1)*100
        self.assertEqual(segment, array('d',self.segmentation))        
        probsImg.setSliceWithoutUpdate(1)
        islPixels = probsImg.getProcessor().getPixels()
        self.assertEqual(array('f',self.pixIslProb),islPixels)
        
    def test_GCClassifier(self):
        self.probImg = self.setImgFromVector(self.pixIslProb, self.imgW, self.imgH)
        GC = Graph_Cut()
        imgGC = GC.processSingleChannelImage(self.probImg, None, float(0.5), float(500), float(500))
        resultPy = imgGC.getProcessor().getPixels()
  
        segmenter = GCSegmenter(self.classifier, self.imgW, self.imgH)
        segmented = segmenter.getSegmentation(self.image)         
        resultJ = segmented.getProcessor().getPixels()
                 
        self.assertEqual(resultJ, resultPy)
        
    def testParticleFilter(self):
        testImgFolder = "data/testImg"  
        binImgName = "binary3particles.tif"
        imgFile = os.path.join(os.getcwd(), testImgFolder, binImgName)
        print imgFile
        img = IJ.openImage(imgFile)
        thrDiam = float(0)
        dXpixNorm = float(1)
        IJ.run(img,"Make Binary","")
        pArea, pa, pb, pAngle = ParticleAnal(img, thrDiam, dXpixNorm)
        filter = ParticleFilter(thrDiam, dXpixNorm)
        img = IJ.openImage(imgFile)
        filter.filterParticles(img)
        jArea = filter.getArea()
        self.assertEqual(array('f',pArea), jArea)
        
        

